@@ -1,4 +1,5 @@
 <?php
+
 declare(strict_types=1);
 /**
  * @copyright Copyright (c) 2019 Joas Schilling <coding@schilljs.com>
@@ -30,7 +31,6 @@ use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Output\OutputInterface;
 
 class Update extends Base {
-
 	use TRenderCommand;
 
 	/** @var CommandService */
@@ -77,7 +77,7 @@ class Update extends Base {
 		;
 	}
 
-	protected function execute(InputInterface $input, OutputInterface $output) {
+	protected function execute(InputInterface $input, OutputInterface $output): int {
 		$id = (int) $input->getArgument('command-id');
 		$cmd = $input->getArgument('cmd');
 		$name = $input->getArgument('name');
@@ -121,5 +121,6 @@ class Update extends Base {
 		$output->writeln('<info>Command updated</info>');
 		$output->writeln('');
 		$this->renderCommands(Base::OUTPUT_FORMAT_PLAIN, $output, [$command]);
+		return 0;
 	}
 }
