@@ -24,6 +24,8 @@ declare(strict_types=1);
 namespace OCA\Talk;
 
 use OCA\Talk\Exceptions\RoomNotFoundException;
+use OCA\Talk\Model\Attendee;
+use OCA\Talk\Service\ParticipantService;
 use OCP\IConfig;
 use OCP\IDBConnection;
 use OCP\DB\QueryBuilder\IQueryBuilder;
@@ -53,6 +55,8 @@ class MatterbridgeManager {
 	private $userManager;
 	/** @var Manager */
 	private $manager;
+	/** @var ParticipantService */
+	private $participantService;
 	/** @var ChatManager */
 	private $chatManager;
 	/** @var IAuthTokenProvider */
@@ -71,6 +75,7 @@ class MatterbridgeManager {
 								IURLGenerator $urlGenerator,
 								IUserManager $userManager,
 								Manager $manager,
+								ParticipantService $participantService,
 								ChatManager $chatManager,
 								IAuthTokenProvider $tokenProvider,
 								ISecureRandom $random,
@@ -83,6 +88,7 @@ class MatterbridgeManager {
 		$this->urlGenerator = $urlGenerator;
 		$this->userManager = $userManager;
 		$this->manager = $manager;
+		$this->participantService = $participantService;
 		$this->chatManager = $chatManager;
 		$this->tokenProvider = $tokenProvider;
 		$this->random = $random;
@@ -297,10 +303,11 @@ class MatterbridgeManager {
 		try {
 			$participant = $room->getParticipant($botUserId);
 		} catch (ParticipantNotFoundException $e) {
-			$room->addUsers([
-				'userId' => $botUserId,
+			$this->participantService->addUsers($room, [[
+				'actorType' => Attendee::ACTOR_USERS,
+				'actorId' => $botUserId,
 				'participantType' => Participant::USER,
-			]);
+			]]);
 		}
 
 		// delete old bot app tokens for this room
@@ -481,17 +488,11 @@ class MatterbridgeManager {
 
 			$content .= '[[gateway.inout]]' . "\n";
 			$content .= sprintf('	account = "%s.%s"', $type, $k) . "\n";
-			if (in_array($type, ['zulip', 'discord', 'xmpp', 'irc', 'slack', 'rocketchat', 'mattermost', 'matrix', 'nctalk'])) {
-				$content .= sprintf('	channel = "%s"', $part['channel']) . "\n";
-				if ($type === 'irc' && $part['password']) {
-					$content .= sprintf('	options = { key = "%s" }', $part['password']) . "\n";
-				}
-				$content .= "\n";
-			} elseif ($type === 'msteams') {
-				$content .= sprintf('	threadId = "%s"', $part['threadid']) . "\n\n";
-			} elseif (in_array($type, ['telegram', 'steam'])) {
-				$content .= sprintf('	chatid = "%s"', $part['chatid']) . "\n\n";
+			$content .= sprintf('	channel = "%s"', $part['channel']) . "\n";
+			if ($type === 'irc' && $part['password']) {
+				$content .= sprintf('	options = { key = "%s" }', $part['password']) . "\n";
 			}
+			$content .= "\n";
 		}
 
 		return $content;
@@ -645,7 +646,7 @@ class MatterbridgeManager {
 	private function sendSystemMessage(Room $room, string $userId, string $message): void {
 		$this->chatManager->addSystemMessage(
 			$room,
-			'users',
+			Attendee::ACTOR_USERS,
 			$userId,
 			json_encode(['message' => $message, 'parameters' => []]),
 			$this->timeFactory->getDateTime(),
